@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2018	PgPool Global Development Group
+ * Copyright (c) 2003-2023	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -457,7 +457,7 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 	else if (MASTER_SLAVE && query_context->is_multi_statement)
 	{
 		/*
-		 * If we are in master/slave mode and we have multi statement query,
+		 * If we are in streaming replication mode and we have multi statement query,
 		 * we should send it to primary server only. Otherwise it is possible
 		 * to send a write query to standby servers because we only use the
 		 * first element of the multi statement query and don't care about the
@@ -488,7 +488,21 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 		/* Should be sent to both primary and standby? */
 		else if (dest == POOL_BOTH)
 		{
-			pool_setall_node_to_be_sent(query_context);
+			if (is_tx_started_by_multi_statement_query())
+			{
+				/*
+				 * If we are in an explicit transaction and the transaction
+				 * was started by a multi statement query, we should send
+				 * query to primary node only (which was supposed to be sent
+				 * to all nodes) until the transaction gets committed or
+				 * aborted.
+				 */
+				pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
+			}
+			else
+			{
+				pool_setall_node_to_be_sent(query_context);
+			}
 		}
 		else if (pool_is_writing_transaction() &&
 				 pool_config->disable_load_balance_on_write == DLBOW_ALWAYS)
@@ -633,6 +647,10 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 			}
 		}
 	}
+	else if (REPLICATION && query_context->is_multi_statement)
+	{
+		pool_setall_node_to_be_sent(query_context);
+	}
 	else if (REPLICATION)
 	{
 		if (pool_config->load_balance_mode &&
@@ -700,24 +718,9 @@ pool_where_to_send(POOL_QUERY_CONTEXT * query_context, char *query, Node *node)
 	}
 
 	/*
-	 * EXECUTE?
+	 * DEALLOCATE or EXECUTE?
 	 */
-	if (IsA(node, ExecuteStmt))
-	{
-		POOL_SENT_MESSAGE *msg;
-
-		msg = pool_get_sent_message('Q', ((ExecuteStmt *) node)->name, POOL_SENT_MESSAGE_CREATED);
-		if (!msg)
-			msg = pool_get_sent_message('P', ((ExecuteStmt *) node)->name, POOL_SENT_MESSAGE_CREATED);
-		if (msg)
-			pool_copy_prep_where(msg->query_context->where_to_send,
-								 query_context->where_to_send);
-	}
-
-	/*
-	 * DEALLOCATE?
-	 */
-	else if (IsA(node, DeallocateStmt))
+	if (IsA(node, DeallocateStmt) || IsA(node, ExecuteStmt))
 	{
 		where_to_send_deallocate(query_context, node);
 	}
@@ -875,8 +878,15 @@ pool_send_and_wait(POOL_QUERY_CONTEXT * query_context,
 		 * Check if some error detected.  If so, emit log. This is useful when
 		 * invalid encoding error occurs. In this case, PostgreSQL does not
 		 * report what statement caused that error and make users confused.
+		 * Also set reset_query_error to true in ERROR case. This does
+		 * anything in normal query processing but when processing reset
+		 * queries, this is important because it might mean DISCARD ALL
+		 * command fails. If so, we need to discard the connection cache so
+		 * that any session object (i.e. named statement) does not remain in
+		 * the last session.
 		 */
-		per_node_error_log(backend, i, string, "pool_send_and_wait: Error or notice message from backend: ", true);
+		if (per_node_error_log(backend, i, string, "pool_send_and_wait: Error or notice message from backend: ", true) == 'E')
+			reset_query_error = true;
 	}
 
 	return POOL_CONTINUE;
@@ -1308,8 +1318,19 @@ static POOL_DEST send_to_where(Node *node, char *query)
 			}
 			/* SAVEPOINT related commands are sent to both primary and standby */
 			else if (is_savepoint_query(node))
+			{
+				if (SL_MODE && is_tx_started_by_multi_statement_query())
+				{
+					/*
+					 * But in streaming replication mode, if a transaction was
+					 * started by a multi statement query, SAVEPOINT should be
+					 * sent to primary because the transaction was started on
+					 * primary only.
+					*/
+					return POOL_PRIMARY;
+				}
 				return POOL_BOTH;
-
+			}
 			/*
 			 * 2PC commands
 			 */
@@ -1468,26 +1489,55 @@ static POOL_DEST send_to_where(Node *node, char *query)
 	return POOL_PRIMARY;
 }
 
+/*
+ * Decide where to send given message.
+ * "node" must be a parse tree of either DEALLOCATE or EXECUTE.
+ */
 static
 void
 where_to_send_deallocate(POOL_QUERY_CONTEXT * query_context, Node *node)
 {
-	DeallocateStmt *d = (DeallocateStmt *) node;
+	DeallocateStmt *d = NULL;
+	ExecuteStmt *e = NULL;
+	char		*name;
 	POOL_SENT_MESSAGE *msg;
 
-	/* DEALLOCATE ALL? */
-	if (d->name == NULL)
+	if (IsA(node, DeallocateStmt))
 	{
-		pool_setall_node_to_be_sent(query_context);
+		d = (DeallocateStmt *) node;
+		name = d->name;
+	}
+	else if (IsA(node, ExecuteStmt))
+	{
+		e = (ExecuteStmt *) node;
+		name = e->name;
 	}
 	else
 	{
-		msg = pool_get_sent_message('Q', d->name, POOL_SENT_MESSAGE_CREATED);
+		ereport(ERROR,
+				(errmsg("invalid node type for where_to_send_deallocate")));
+		return;
+	}
+
+	/* DEALLOCATE ALL? */
+	if (d && (name == NULL))
+	{
+		/* send to all backend node */
+		pool_setall_node_to_be_sent(query_context);
+		return;
+	}
+
+	/* ordinary DEALLOCATE or EXECUTE */
+	else
+	{
+		/* check if message was created by SQL PREPARE */
+		msg = pool_get_sent_message('Q', name, POOL_SENT_MESSAGE_CREATED);
 		if (!msg)
-			msg = pool_get_sent_message('P', d->name, POOL_SENT_MESSAGE_CREATED);
+			/* message may be created by Parse message */
+			msg = pool_get_sent_message('P', name, POOL_SENT_MESSAGE_CREATED);
 		if (msg)
 		{
-			/* Inherit same map from PREPARE or PARSE */
+			/* Inherit same map from PREPARE or Parse */
 			pool_copy_prep_where(msg->query_context->where_to_send,
 								 query_context->where_to_send);
 
@@ -1495,8 +1545,33 @@ where_to_send_deallocate(POOL_QUERY_CONTEXT * query_context, Node *node)
 			query_context->load_balance_node_id = msg->query_context->load_balance_node_id;
 		}
 		else
-			/* prepared statement was not found */
-			pool_setall_node_to_be_sent(query_context);
+		{
+			/*
+			 * prepared statement was not found.
+			 * There are two cases when this could happen.
+			 * (1) mistakes by client. In this case backend will return ERROR
+			 * anyway.
+			 * (2) previous query was issued as multi-statement query. e.g.
+			 * SELECT 1\;PREPARE foo AS SELECT 1;
+			 * In this case pgpool does not know anything about the prepared
+			 * statement "foo".
+			 */
+			if (SL_MODE)
+			{
+				/*
+				 * In streaming replication or logical replication, sent to
+				 * primary node only.
+				 */
+				pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
+			}
+			else
+			{
+				/*
+				 * In other mode, sent to all node.
+				 */
+				pool_setall_node_to_be_sent(query_context);
+			}
+		}
 	}
 }
 
