@@ -4,7 +4,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2023	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -2048,3 +2048,45 @@ pool_temp_tables_dump(void)
 	}
 #endif
 }
+
+/*
+ * Return true if an explicit transaction has been started by a
+ * multi-statement-query
+ */
+bool
+is_tx_started_by_multi_statement_query(void)
+{
+	if (!session_context)
+		ereport(ERROR,
+				(errmsg("is_tx_started_by_multi_statement_query: session context is not initialized")));
+
+	return session_context->is_tx_started_by_multi_statement;
+}
+
+/*
+ * Remember that an explicit transaction has been started by a
+ * multi-statement-query
+ */
+void
+set_tx_started_by_multi_statement_query(void)
+{
+	if (!session_context)
+		ereport(ERROR,
+				(errmsg("set_tx_started_by_multi_statement_query: session context is not initialized")));
+
+	session_context->is_tx_started_by_multi_statement = true;
+}
+
+/*
+ * Forget that an explicit transaction has been started by a
+ * multi-statement-query
+ */
+void
+unset_tx_started_by_multi_statement_query(void)
+{
+	if (!session_context)
+		ereport(ERROR,
+				(errmsg("unset_tx_started_by_multi_statement_query: session context is not initialized")));
+
+	session_context->is_tx_started_by_multi_statement = false;
+}
